import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

/**
  *
  * Beschreibung
  *
  * @version 1.0 vom 30.04.2012
  * @author
  */

public class Vigenere extends JFrame {
  // Anfang Attribute
  private JTextField jTextField1 = new JTextField();
  private JTextField jTextField2 = new JTextField();
  private JTextField jTextField3 = new JTextField();
  private JLabel jLabel1 = new JLabel();
  private JLabel jLabel2 = new JLabel();
  private JLabel jLabel3 = new JLabel();
  private JButton jButton1 = new JButton();
  private JButton jButton2 = new JButton();
  // Ende Attribute
  
  public Vigenere(String title) {
    // Frame-Initialisierung
    super(title);
    setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
    int frameWidth = 292; 
    int frameHeight = 304;
    setSize(frameWidth, frameHeight);
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    int x = (d.width - getSize().width) / 2;
    int y = (d.height - getSize().height) / 2;
    setLocation(x, y);
    setResizable(false);
    Container cp = getContentPane();
    cp.setLayout(null);
    // Anfang Komponenten
    
    jTextField1.setBounds(8, 8, 273, 33);
    cp.add(jTextField1);
    jTextField2.setBounds(72, 72, 145, 33);
    cp.add(jTextField2);
    jTextField3.setBounds(8, 136, 273, 33);
    cp.add(jTextField3);
    jLabel1.setBounds(120, 40, 49, 19);
    jLabel1.setText("Klartext");
    cp.add(jLabel1);
    jLabel2.setBounds(112, 104, 61, 19);
    jLabel2.setText("Schlssel");
    cp.add(jLabel2);
    jLabel3.setBounds(112, 168, 68, 19);
    jLabel3.setText("Geheimtext");
    cp.add(jLabel3);
    jButton1.setBounds(8, 216, 121, 41);
    jButton1.setText("Kodieren");
    jButton1.setMargin(new Insets(2, 2, 2, 2));
    jButton1.addActionListener(new ActionListener() { 
      public void actionPerformed(ActionEvent evt) { 
        jButton1_ActionPerformed(evt);
      }
    });
    cp.add(jButton1);
    jButton2.setBounds(160, 216, 121, 41);
    jButton2.setText("Dekodieren");
    jButton2.setMargin(new Insets(2, 2, 2, 2));
    jButton2.addActionListener(new ActionListener() { 
      public void actionPerformed(ActionEvent evt) { 
        jButton2_ActionPerformed(evt);
      }
    });
    cp.add(jButton2);
    // Ende Komponenten
    
    setVisible(true);
  }
  
  
  // Anfang Methoden
  
  private boolean isValid(char z) {
    return (z >= 'A') && (z <= 'Z');
  }
  
  /*
   * Baut aus dem bergebenen String key einen gltigen
   * Vigenre-Schlssel, d.h. einen String, der ausschlielich
   * aus Grobuchstaben besteht ['A'...'Z'], und gibt diesen
   * Schlssel zurck
   */
  private String getValidVigenereKey(String key) {
    key = key.toUpperCase();
    String newKey = "";
    for (int i = 0; i < key.length(); i++) {
      if (isValid(key.charAt(i))) {
        newKey = newKey + key.charAt(i);
      } // end of if
    } // end of for
    return newKey;
  }
  
  private int getDisplacement(char v) {
    return (int) v - (int) 'A';
  }
  
  /*
   * Implementiert die "Caesar-Verschiebung" des Alphabets:
   * es wird derjenige Buchstabe c berechnet und zurckgegeben,
   * der im Alphabet v Stellen hinter dem bergebenen Original-
   * Buchstaben z steht.
   */
  private char caesar(char z, int v) {
    int a = (int) z;        // Typ-Umandlung char z ==> int a
    int n = a - (int) 'A';  // ASCII-Nr.-Bereich verschieben nach [0..25]
    int m = (n + v) % 26;   // Eigentliche Verschlsselung
    int b = m + (int) 'A';  // zurckverschieben in originalen ASCII-Nr.-Bereich
    char c = (char) b;      // Typ-Umwandlung int b ==> char c
    return c;               // gib c zurck
  }
  
  /*
   * Berechnet aus dem bergebenen "Verschlsselungs-Schlssel" key den
   * passenden (d.h. "komplementren") "Entschlsselungs-Schlssel"
   */
  private String getComplementKey(String key) {
    String compKey = "";
    for (int i = 0; i < key.length(); i++) {
      int d = getDisplacement(key.charAt(i));
      int cd = (26 - d) % 26;
      char cc = (char) ( ((int) 'A') + cd);
      compKey = compKey + cc;
    } // end of for
    return compKey;
  }

  private String encrypt(String text, String key) {
    String encrypted = "";              // verschlsselter Text, zunchst leer
    int max = text.length() - 1;        // Index des letzten Klartext-Zeichens
    int i = 0;                          // i-ter Buchstabe im Klartext
    int n = 0;                          // n-ter Buchstabe im Schlssel
    while (i <= max) {                  // solange i auf Text-Buchstaben zeigt
      char b = text.charAt(i);          // nimm Text-Buchstabe b an der Stelle i
      char c = key.charAt(n);           // nimm Key-Buchstabe c an der Stelle n
      if (isValid(b) && isValid(c)) {   // wenn beide Buchstaben gltig sind:
        int verschieb = getDisplacement(c);
        char v = caesar(b, verschieb);  // verschlsseln und an den...
        encrypted = encrypted + v;      // ...verschlsselten Text anfgen
        n++;                            // Schlssel-Index n erhhen...
        if (n >= key.length()) {        // ... bzw. auf 0 zurcksetzen, wenn
          n = 0;                        //     das Schlsselende erreicht ist
        } // end of if
      };
      i++;                              // c um eins erhoehen
    }
    return encrypted;    // Verschlsselten Text zurckliefern
  }
  
  private String decrypt(String geheimtext, String schluessel) {
    String decSchluessel = getComplementKey(schluessel);
    String decrStr = encrypt(geheimtext, decSchluessel);
    return decrStr;
  }
  
  public void jButton1_ActionPerformed(ActionEvent evt) {
    String plainText = jTextField1.getText().toUpperCase();
    String key = getValidVigenereKey(jTextField2.getText());
    jTextField2.setText(key);
    String encrText = encrypt(plainText, key);
    jTextField3.setText(encrText);
  }
  
  public void jButton2_ActionPerformed(ActionEvent evt) {
    String cryptText = jTextField3.getText().toUpperCase();
    String key = getValidVigenereKey(jTextField2.getText());
    jTextField2.setText(key);
    String plainText = decrypt(cryptText, key);
    jTextField1.setText(plainText);
  }
  
  public static void main(String[] args) {
    new Vigenere("Vigenre-Verschlsselung");
  }
  
  // Ende Methoden
}
